/*
    kDriver - PC keyboard driver for the TI-89
    Copyright (C) 2003  Sbastien Bourdeauducq

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#define _NO_INCLUDE_PATCH

#include "config.h"

#include <events.h>
#include <version.h>

#include "comm.h"
#include "main.h"

#if __TIGCC__==0 && __TIGCC_MINOR__< 94 && __TIGCC_BETA__ < 19
  #warning Old TIGCC Version -- Check tigcc.ticalc.org for updates
#endif

#define CM_EVHK_IS_PRESENT 0x7FFA
#define CM_EVHK_CAN_DELETE 0x7FFF
#define CM_EVHK_DELETE 0x7FFE
#define CM_EVHK_ABOUT 0x7FFC
#define CM_EVHK_INSTALL 0x7FFB

#define TSR_NAME "kDriver "

void Install_TSR(void);
void EventHandler(EVENT *ev);

asm(".section _stl1 | First library startup section. This is what the executable starts with.

.ascii \"EvHk" TSR_NAME "\"
.globl        gpOldHandler
gpOldHandler:
.long 0
__jmp__Event_Hook__:
jmp Install_TSR:l

.global __calculator
__calculator:
.word 0");


extern EVENT_HANDLER gpOldHandler;

void Install_TSR(void)
{
  extern char __jmp__Event_Hook__;
  char *ptr = &__jmp__Event_Hook__;

  *(unsigned long*)&ptr[2]=(unsigned long)EventHandler;

  asm volatile("
  /* Start Of Calculator Detection Code -- From TIGCC Sources */
      move.l 0xC8,%%a0
      /* Calculator Detection */
      moveq #1,%%d0
      move.l %%a0,%%d1
      and.l #0x400000,%%d1
      jbne __calc_in_d0__
      clr.w %%d0
      move.l (%%a0,0x2F*4),%%a1 /* ScrRect */
      cmp.b #200,2(%%a1)
      jbcs __calc_in_d0__
      moveq #3,%%d0
    __calc_in_d0__:
      lea __calculator(%%pc),%%a1
      move.w %%d0,(%%a1)
  /* End Of Calculator Detection Code -- From TIGCC Sources */"
  :
  :
  : "a0", "a1", "d1");

  link_init();
}

void EventHandler(EVENT *ev)
{
  EVENT_HANDLER OldHook;

  void *__saved_a5__;
  asm("/* START: Support for OPTIMIZE_ROM_CALLS */");
  asm("move.l %%a5,%0":"=p"(__saved_a5__));
  asm("move.l 0xC8,%a5");
  asm("/* END: Support for OPTIMIZE_ROM_CALLS */");


  /////////////////////////////////////
  // Event Handler Code Starts Here  //
  /////////////////////////////////////

  switch(ev->Type) {
    case CM_EVHK_DELETE:
      link_done();
      break;
    case CM_IDLE:
      poll_kbd();
      break;
    default:
      break;
  }

  /////////////////////////
  //End of Event Handler //
  /////////////////////////


  OldHook = gpOldHandler;

  if(OldHook)
  {
    asm("                 /* This inline asm takes the place of OldHook(ev);
                          the difference is that a2 will also contain ev
                          as a parameter to support TEOS. */
    move.l %0,%%a2
    move.l %%a2,-(%%sp)
    move.l %1,%%a0
    jsr (%%a0)
    addq.l #4,%%sp
    "::"p"(ev),"g"(OldHook):"a0","a2");
  }

  asm("/* START: Support for OPTIMIZE_ROM_CALLS */");
  asm("move.l %0, %%a5	/* support for OPTIMIZE_ROM_CALLS */"::"p"(__saved_a5__));
  asm("/* END: Support for OPTIMIZE_ROM_CALLS */");
}
