/*
    kDriver - PC keyboard driver for the TI-89
    Copyright (C) 2003  Sbastien Bourdeauducq

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#define _NO_INCLUDE_PATCH

#include "config.h"

#include <compat.h>
#include <kbd.h>

#include "comm.h"

#define SCROLLLOCK 0b00000001
#define NUMLOCK 0b00000010
#define CAPSLOCK 0b00000100

unsigned char led_status;
unsigned char shift, ctrl, alt, altgr;

#define numlock ((led_status & NUMLOCK) == NUMLOCK)

unsigned char caps()
{
  if((led_status & CAPSLOCK) == CAPSLOCK) return !shift; else return shift;
}

void poll_kbd()
{
  short c, c2, c3;
  unsigned char b;

  c = receive_byte();
  if(c == -1) return;
  switch(c) {
    case 0x15: if(caps()) pushkey('A'); else pushkey('a'); break;
    case 0x1d: if(caps()) pushkey('Z'); else pushkey('z'); break;
    case 0x24: if(altgr) pushkey(16472); else { if(caps()) pushkey('E'); else pushkey('e'); } break;
    case 0x2d: if(caps()) pushkey('R'); else pushkey('r'); break;
    case 0x2c: if(caps()) pushkey('T'); else pushkey('t'); break;
    case 0x35: if(caps()) pushkey('Y'); else pushkey('y'); break;
    case 0x3c: if(caps()) pushkey('U'); else pushkey('u'); break;
    case 0x43: if(altgr) pushkey(151); else { if(caps()) pushkey('I'); else pushkey('i'); } break;
    case 0x44: if(caps()) pushkey('O'); else pushkey('o'); break;
    case 0x4d: if(caps()) pushkey('P'); else pushkey('p'); break;
    case 0x1c: if(caps()) pushkey('Q'); else pushkey('q'); break;
    case 0x1b: if(caps()) pushkey('S'); else pushkey('s'); break;
    case 0x23: if(caps()) pushkey('D'); else pushkey('d'); break;
    case 0x2b: if(caps()) pushkey('F'); else pushkey('f'); break;
    case 0x34: if(caps()) pushkey('G'); else pushkey('g'); break;
    case 0x33: if(caps()) pushkey('H'); else pushkey('h'); break;
    case 0x3b: if(caps()) pushkey('J'); else pushkey('j'); break;
    case 0x42: if(caps()) pushkey('K'); else pushkey('k'); break;
    case 0x4b: if(caps()) pushkey('L'); else pushkey('l'); break;
    case 0x4c: if(caps()) pushkey('M'); else pushkey('m'); break;
    case 0x1a: if(caps()) pushkey('W'); else pushkey('w'); break;
    case 0x22: if(ctrl) pushkey(20480); else { if(caps()) pushkey('X'); else pushkey('x'); } break;
    case 0x21: if(ctrl) pushkey(24576); else { if(caps()) pushkey('C'); else pushkey('c'); } break;
    case 0x2a: if(ctrl) pushkey(16648); else { if(caps()) pushkey('V'); else pushkey('v'); } break;
    case 0x32: if(caps()) pushkey('B'); else pushkey('b'); break;
    case 0x31: if(caps()) pushkey('N'); else pushkey('n'); break;
    case 0x70: if(numlock || shift) pushkey('0'); else pushkey(KEY_INS); break;
    case 0x71: if(numlock || shift) pushkey('.'); else pushkey(KEY_DIAMOND+KEY_BACKSPACE); break;
    case 0x69: if(numlock || shift) pushkey('1'); else pushkey(4440); break;
    case 0x72: if(numlock || shift) pushkey('2'); else pushkey(KEY_DOWN); break;
    case 0x7a: if(numlock || shift) pushkey('3'); else pushkey(4436); break;
    case 0x6b: if(numlock || shift) pushkey('4'); else pushkey(KEY_LEFT); break;
    case 0x73: if(numlock || shift) pushkey('5'); break;
    case 0x74: if(numlock || shift) pushkey('6'); else pushkey(KEY_RIGHT); break;
    case 0x6c: if(numlock || shift) pushkey('7'); else pushkey(4434); break;
    case 0x75: if(numlock || shift) pushkey('8'); else pushkey(KEY_UP); break;
    case 0x7d: if(numlock || shift) pushkey('9'); else pushkey(4433); break;
    case 0x7c: pushkey('*'); break;
    case 0x7b: if(ctrl) pushkey(173); else pushkey('-'); break;
    case 0x79: pushkey('+'); break;
    case 0x61: if(shift) { if(alt) pushkey(16430); else pushkey('>'); } else { if(alt) pushkey(16432); else pushkey('<'); } break;
    case 0x0d: if(alt) pushkey(KEY_SWITCH); else pushkey(258); break;
    case 0x0E: pushkey(277); break;
    case 0x16: if(shift) pushkey('1'); else pushkey('&'); break;
    case 0x1E: if(shift) pushkey('2'); else { if(altgr) pushkey('~'); else pushkey('e'); } break; /* FIXME: using e instead of  */
    case 0x26: if(shift) pushkey('3'); else { if(altgr) pushkey('#'); else pushkey('"'); } break;
    case 0x25: if(shift) pushkey('4'); else { if(altgr) pushkey('{'); else pushkey('\''); } break;
    case 0x2E: if(shift) pushkey('5'); else { if(altgr) pushkey('['); else pushkey('('); } break;
    case 0x36: if(shift) pushkey('6'); else { if(altgr) pushkey('|'); else pushkey('-'); } break;
    case 0x3D: if(shift) pushkey('7'); else { if(altgr) pushkey('\''); else pushkey('e'); } break; /* FIXME: using e instead of  */
    case 0x3E: if(shift) pushkey('8'); else { if(altgr) pushkey('\\'); else pushkey('_'); } break;
    case 0x46: if(shift) pushkey('9'); else { if(altgr) pushkey(94); else pushkey('c'); } break; /* FIXME: using c instead of  */
    case 0x45: if(shift) pushkey('0'); else { if(altgr) pushkey('@'); else pushkey('a'); } break; /* FIXME: using a instead of  */
    case 0x4E: if(shift) pushkey(''); else { if(altgr) pushkey(']'); else pushkey(')'); } break;
    case 0x55: if(shift) pushkey('+'); else { if(altgr) pushkey('}'); else pushkey('='); } break;
    case 0x3a: if(shift) pushkey('?'); else pushkey(','); break;
    case 0x41: if(shift) pushkey('.'); else pushkey(';'); break;
    case 0x49: if(shift) pushkey('/'); else pushkey(':'); break;
    case 0x4a: if(shift) pushkey(190); else pushkey('!'); break;
    case 0x54: if(shift) pushkey(18); else pushkey(94); break;
    case 0x5b: if(shift) pushkey(136); else pushkey(140); break;
    case 0x52: pushkey('u'); break; /* FIXME: using u instead of  */
    case 0x29: pushkey(' '); break;
    case 0x5a: if(alt) pushkey(KEY_DIAMOND+KEY_ENTER); else { if(ctrl) pushkey(KEY_ENTRY); else pushkey(KEY_ENTER); } break;
    case 0x66: if(ctrl) pushkey(KEY_CLEAR); else pushkey(KEY_BACKSPACE); break;
    case 0x76: if(ctrl) pushkey(KEY_QUIT); else pushkey(KEY_ESC); break;
    case 0x05: pushkey(KEY_F1); break;
    case 0x06: pushkey(KEY_F2); break;
    case 0x04: pushkey(KEY_F3); break;
    case 0x0c: pushkey(KEY_F4); break;
    case 0x03: pushkey(KEY_F5); break;
    case 0x0b: pushkey(KEY_F6); break;
    case 0x83: pushkey(KEY_F7); break;
    case 0x0a: pushkey(KEY_F8); break;
    case 0x01: pushkey(16652); break;
    case 0x09: pushkey(16653); break;
    case 0x78: pushkey(16654); break;
    case 0x07: pushkey(16656); break;
    case 0x7e: /* Scroll Lock */ break;
    case 0xe1:
      if(receive_byte() != 0x14) break;
      if(receive_byte() != 0x77) break;
      if(receive_byte() != 0xE1) break;
      if(receive_byte() != 0xF0) break;
      if(receive_byte() != 0x14) break;
      if(receive_byte() != 0xF0) break;
      if(receive_byte() != 0x77) break;
      /* Pause */
      break;
    case 0x11:
      alt = TRUE;
      break;
    case 0x14:
      ctrl = TRUE;
      break;
    case 0x12:
    case 0x59:
      shift = TRUE;
      break;
    case 0xF0:
      c2 = receive_byte();
      switch(c2) {
        case 0x11:
          alt = FALSE;
          break;
        case 0x14:
          ctrl = FALSE;
          break;
        case 0x12:
        case 0x59:
          shift = FALSE;
          break;
        default:
          break;
      }
      break;
    case 0x58:
      b = led_status;
      b ^= CAPSLOCK;
      if(send_byte(0xed)) break;
      if(receive_byte() != 0xfa) break;
      if(send_byte(b)) break;
      led_status = b;
      break;
    case 0x77:
      b = led_status;
      b ^= NUMLOCK;
      if(send_byte(0xed)) break;
      if(receive_byte() != 0xfa) break;
      if(send_byte(b)) break;
      led_status = b;
      break;
    case 0xAA:
      shift = FALSE;
      ctrl = FALSE;
      alt = FALSE;
      altgr = FALSE;
      led_status = NUMLOCK;
      if(send_byte(0xed)) break;
      if(receive_byte() != 0xfa) break;
      send_byte(led_status);
      break;
    case 0xe0:
      c2 = receive_byte();
      switch(c2) {
        case 0x11:
          altgr = TRUE;
          break;
        case 0x14:
          ctrl = TRUE;
          break;
        case 0x1f:
        case 0x27: pushkey(KEY_APPS); break; /* Windoze keys */
        case 0x2f: pushkey(278); break; /* Menu key */
        case 0x75: if(shift) pushkey(KEY_SHIFT+KEY_UP); else pushkey(KEY_UP); break;
        case 0x72: if(shift) pushkey(KEY_SHIFT+KEY_DOWN); else pushkey(KEY_DOWN); break;
        case 0x6B: if(shift) pushkey(KEY_SHIFT+KEY_LEFT); else pushkey(KEY_LEFT); break;
        case 0x74: if(shift) pushkey(KEY_SHIFT+KEY_RIGHT); else pushkey(KEY_RIGHT); break;
        case 0x70: pushkey(KEY_INS); break;
        case 0x71: pushkey(KEY_DIAMOND+KEY_BACKSPACE); break;
        case 0x6c: pushkey(4434); break;
        case 0x69: pushkey(4440); break;
        case 0x7d: if(shift) pushkey(16721); else pushkey(4433); break;
        case 0x7a: if(shift) pushkey(16724); else pushkey(4436); break;
        case 0x4a: pushkey('/'); break;
        case 0x5a: if(alt) pushkey(KEY_DIAMOND+KEY_ENTER); else { if(ctrl) pushkey(KEY_ENTRY); else pushkey(KEY_ENTER); } break;
        case 0xf0:
          c3 = receive_byte();
          switch(c3) {
            case 0x11:
              altgr = FALSE;
              break;
            case 0x14:
              ctrl = FALSE;
              break;
            default:
              break;
          }
          break;
        default: break;
      }
    default: break;
  }
}

